/* main.c */
/* THE QUICK AND DIRTY COMMUNICATION TOOL */

/* Limitations :
   Pipe implemetation : does not cross the network
   Drawing area cleared if resized
   Drawing area cleared if overlapped
*/

#include <gtk/gtk.h>

#include <stdio.h> /* sprintf */ 
#include <stdlib.h> /* atoi */
#include <unistd.h> /* open, close ... */
#include <fcntl.h> /* Attributs O_RDONLY ... */
#include <sys/types.h> /* mkfifo () */
#include <sys/stat.h> /* mkfifo () */
#include <sys/wait.h> /* wait */
#include <string.h>

#include "interface.h"
#include "support.h"
#include "callbacks.h"


int PipeRead = -1;
int PipeWrite = -1;
GtkWidget *text;
GtkWidget *DrawingArea;
GdkColor color_a;
GdkColor color_b;

int theId;

GdkGC *gc = NULL; /* Le contexte graphique */

int main (int argc, char *argv[]) {

  char nameOut [80];
  char nameIn [80];
  pid_t pid;
  int status;
  int argcp;
  char **argvp;
  unsigned long thepid;

  if (argc != 3) {
    printf ("Syntaxe : %s --display <un_display>\n", argv [0]);
    exit (1);
  }

  thepid = getpid();
  sprintf (nameOut, "/tmp/talk_1_%ld", thepid); /* Ecriture */
  sprintf (nameIn, "/tmp/talk_2_%ld", thepid); /* Lecture */

  /* Creation d'un fils */
  pid = fork ();
  if ( pid == (pid_t) (-1)) {
    printf ("Erreur : Echec du fork\n");
    exit (-1);
  }
  else if ( pid == 0) { /* Demarrage du fils */ 
    if (gtk_init_check (&argc, &argv) == TRUE) {
      mkfifo (nameOut, 0666);
      mkfifo (nameIn, 0666);
      PipeRead = open (nameIn, O_RDONLY | O_NONBLOCK); /* Ecriture Master->Slave */
      PipeWrite = open (nameOut, O_WRONLY);
      InstanciateTask (0);

      remove (nameOut);
      remove (nameIn);
      exit (0);
    }
    else {
      exit (1);
    }
  }

  sleep (1); /* On attend un peu pour avoir les infos du fils */
  waitpid (pid, &status, WNOHANG);
	if (0) {
/*  if (status) { */
    printf ("Error : Unable to open remote display %s\n", argv [2]);
    return 1;
  }

  argvp = (char **) calloc (2, sizeof (char *)); /* 2 pointeurs */
  argvp [0] = (char *) calloc (256, sizeof (char));
  argvp [1] = 0;
  strcpy (argvp [0], argv [0]);

  argcp = 1;
  gtk_init (&argcp, &argvp);
  PipeRead = open (nameOut, O_RDONLY | O_NONBLOCK);
  PipeWrite = open (nameIn, O_WRONLY);

  SendCaller ();
  InstanciateTask (1);

  wait (&status); /* Suppression de la table des process */
  return 0;
}

int InstanciateTask (int id) {  
  GtkWidget *Window;
  int idle_task;
  GdkColormap *colormap; /* La color map par defaut */
  Window = create_TalkWindow ();
  gtk_widget_show (Window);

  text = GTK_WIDGET (lookup_widget (Window, "TextRemote"));
  DrawingArea = lookup_widget (GTK_WIDGET (Window), "DrawingArea");
  colormap = gdk_window_get_colormap (DrawingArea->window);
  gc = gdk_gc_new (DrawingArea->window);
  
  color_a.red = 0;
  color_a.green = 0;
  color_a.blue = 65535;
  
  color_b.red = 65535;
  color_b.green = 0;
  color_b.blue = 0;

  gdk_color_alloc (colormap, &color_a);
  gdk_color_alloc (colormap, &color_b);

  theId = id;
  
  /* Traitement des donnees en provenance des pipe de com */
  idle_task = gtk_idle_add (Listen, 0);

  
  gtk_main ();
  gtk_idle_remove (idle_task);
  gdk_gc_destroy (gc);

  close (PipeRead);
  close (PipeWrite);
  return 0;
}
